LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;


--  Entity Declaration

ENTITY debounce IS
  GENERIC(
    CLK_FREQ  : INTEGER := 50e6
  );
	PORT(
    CLK       : IN  STD_LOGIC;
    SIG_IN    : IN  STD_LOGIC;
    SIG_OUT   : OUT STD_LOGIC
  );

END debounce;

--  Architecture Body

ARCHITECTURE debounce_architecture OF debounce IS

  signal reg0 : STD_LOGIC;
  signal reg1 : STD_LOGIC;
  
--  constant max_count  : INTEGER := CLK_FREQ;  -- 1 s
  constant max_count  : INTEGER := CLK_FREQ/10;  -- 100 ms
  
  signal count      : INTEGER := 0;
  signal count_rst  : STD_LOGIC := '1';
  

BEGIN

  count_rst <= reg0 xor reg1;

  debounce_proc : process(clk)
  begin
    
    if(clk'event and clk = '1') then
      reg0  <= SIG_IN;
      reg1  <= reg0;
      if (count_rst = '1') then
        count <= 0;
      elsif (count = max_count) then
        SIG_OUT <= reg1;
        count <= 0;
      else
        count <= count + 1;
      end if;
    end if;
  
  end process;


END debounce_architecture;